// eventually we can have a base adapter class
var AccuWeatherAdapter, FeedError;

(function() {
    var SEARCH_URL = "http://forecastfox3.accuweather.com/adcbin/forecastfox3/city-find.asp?", CC_URL = "http://forecastfox3.accuweather.com/adcbin/forecastfox3/current-conditions.asp?location=${location}&metric=0&langId=${lang}", FORECAST_URL = "http://forecastfox3.accuweather.com/adcbin/forecastfox3/forecast-data.asp?location=${location}&metric=0&langId=${lang}", COORD_ARGS = "latitude=${lat}&longitude=${lon}", LOCID_ARGS = "location=${loc}", FeedError = {
        CONNECTION: 1,
        LOCATION_NOT_FOUND: 2,
        UNKNOWN: 3
    };
    function get_find_location_url(query) {
        var args = {}, argArr = [], url = SEARCH_URL;
        "location" in query && (args.location = query.location), "latlng" in query && (args.latitude = query.latlng.lat, args.longitude = query.latlng.lng);
        for (var x in args) argArr.push(x + "=" + encodeURIComponent(args[x]));
        return url + argArr.join("&");
    }
    AccuWeatherAdapter = Class.extend({
        init: function(logging, i18n, templator) {
            this._logger = logging.getLogger("accuweather-adapter"), this._i18n = i18n, this._t = templator, this._feed_xhr = {
                cc: null,
                forecast: null
            };
        },
        find_location: function(query, search_id, callback, errback) {
            var url = get_find_location_url(query), self = this;
            this._logger.debug("Looking up location: " + url);
            var deferred = $.ajax({
                url: url,
                type: "GET",
                cache: !1,
                timeout: 1e4
            });
            deferred.error(function(xmlHttpReq, status, error) {
                self._logger.error("failed to find location: " + (error || "") + " " + (status || ""));
                try {
                    analytics.trackEvent("Errors", "LocationSearch", "Coords");
                } catch (e) {
                    self._logger.debug("ERROR: " + e);
                }
                var errorType = FeedError.UNKNOWN;
                status == "timeout" && (errorType = FeedError.CONNECTION), errback(search_id, errorType);
            }), deferred.success(function(data, status) {
                self._logger.debug("find_location success!" + data);
                var json = $.xml2json(data);
                self._logger.debug("json: " + JSON.stringify(json));
                if (!(!(json.length < 1 || typeof json != "object") && "citylist" in json && "location" in json.citylist)) {
                    errback(search_id, FeedError.LOCATION_NOT_FOUND);
                    return;
                }
                var locs = json.citylist.location;
                locs ? callback(search_id, locs.location, locs.city, locs.state) : errback(search_id, FeedError.LOCATION_NOT_FOUND);
            });
        },
        fetch_feed_data: function(code, feed_type) {
            var self = this, lang_code = this._i18n.accucode();
            lang_code === null && (lang_code = this._i18n.accucode("en"));
            var url = this._t.render(feed_type == "cc" ? CC_URL : FORECAST_URL, {
                location: encodeURIComponent(code),
                lang: encodeURIComponent(lang_code)
            });
            this._logger.debug("Fetching data for " + code + " from " + url), this._feed_xhr[feed_type] && this._feed_xhr[feed_type].abort();
            var dfr = $.Deferred();
            return navigator.onLine ? (this._feed_xhr[feed_type] = $.ajax({
                url: url,
                type: "GET",
                timeout: 2e4
            }).fail(function(xmlHttpReq, status, error) {
                self._logger.error("feed update (" + code + "," + feed_type + ") failed: " + (status || "") + " " + (error || "")), self._feed_xhr[feed_type] = null;
                try {
                    analytics.trackEvent("Errors", "Feed", code);
                } catch (e) {
                    self._logger.debug("ERROR: " + e);
                }
                var errorType = FeedError.UNKNOWN;
                status == "timeout" && (errorType = FeedError.CONNECTION), dfr.reject(errorType);
            }).done(function(data, status) {
                self._logger.debug("fetch feed data success! for " + code + "," + feed_type), self._feed_xhr[feed_type] = null;
                try {
                    var json = $.xml2json(data);
                    if (!json) {
                        dfr.reject(FeedError.UNKNOWN);
                        return;
                    }
                    if ("error" in json) {
                        json.error == "Invalid location, or no location found" ? dfr.reject(FeedError.LOCATION_NOT_FOUND) : dfr.reject(FeedError.UNKNOWN);
                        return;
                    }
                    json.locale = self._i18n.locale();
                } catch (e) {
                    self._logger.exception("feed data (" + code + ") failed parse", e), dfr.reject(FeedError.UNKNOWN);
                    return;
                }
                dfr.resolve(json);
            }), dfr) : dfr.reject(FeedError.CONNECTION);
        }
    });
})();