/*******************************************************************************
 * Object used to abstract away the different storage mechanisms that we may
 * use to store data.  All values in the data store are stored as strings.
 * This script should be included in such a way as to not pollute
 * the global context.
 *
 * @see /content/logging.js
 * @see /content/external/class.js
 * @version 1.0
 ******************************************************************************/var DataStore = Class.extend({
    _logger: null,
    _storage: null,
    init: function(logging) {
        this._logger = logging.getLogger("datastore");
        var scope = {}, loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService(Components.interfaces.mozIJSSubScriptLoader);
        loader.loadSubScript("chrome://forecastfox/content/sqlite.js", scope), this._storage = new scope.SQLite(logging), this._logger.debug("initialized");
    },
    keys: function() {
        var results = this._storage.query("keys", "SELECT key FROM datastore");
        this._logger.debug("getting " + results.length + " keys");
        var keys = [];
        for (var i = 0; i < results.length; i++) keys.push(results[i].key);
        return keys.sort(), keys;
    },
    get: function(key, parse) {
        var results = this._storage.query("get", "SELECT value FROM datastore WHERE key=:key", {
            key: key
        }), value = results.length ? results[0].value : null;
        if (value && parse) try {
            return JSON.parse(value);
        } catch (e) {
            return this._logger.exception("json parsing failed for " + key, e), null;
        }
        return this._logger.debug("data retrieved for " + key), value;
    },
    set: function(key, value, stringify) {
        if (stringify) try {
            value = JSON.stringify(value);
            if (value == "{}") return this._logger.error("json stringify failed for " + key), !1;
        } catch (e) {
            return this._logger.exception("json stringify failed for " + key, e), !1;
        }
        if (typeof value != "string") return this._logger.error("value is not a string for " + key), !1;
        var success = !1;
        return this._has(key) ? success = this._storage.execute("update", "UPDATE datastore SET value=:value WHERE key=:key;", {
            key: key,
            value: value
        }) : success = this._storage.execute("insert", "INSERT INTO datastore (key, value) VALUES(:key, :value);", {
            key: key,
            value: value
        }), success ? (this._logger.debug("data stored for " + key), !0) : (this._logger.error("storing data failed for " + key), !1);
    },
    remove: function(key) {
        this._storage.execute("remove", "DELETE FROM datastore WHERE key=:key;", {
            key: key
        }), this._logger.debug("data removed for " + key);
    },
    clear: function() {
        this._storage.execute("clear", "DELETE FROM datastore;"), this._logger.debug("data store cleared");
    },
    all: function() {
        this._logger.debug("retrieving all key/value pairs in the data store");
        var obj = {}, results = this._storage.query("all", "SELECT * FROM datastore;");
        for (var i = 0; i < results.length; i++) {
            var row = results[i];
            obj[row.key] = row.value;
        }
        return obj;
    },
    ingest: function(obj) {
        for (var key in obj) {
            var value = obj[key];
            this.set(key, value), this._logger.debug("ingesting " + value + " for " + key + " key");
        }
    },
    _has: function(key) {
        var results = this._storage.query("has", "SELECT key FROM datastore WHERE key=:key", {
            key: key
        });
        return this._logger.debug("checking if key " + key + " exists"), results.length > 0;
    }
});