// options.js exposes these names
var get_options, set_options, show_saved_message;

(function() {
    var URL_REGEX = /^https?:\/\/.+$/, previous_options = {};
    get_options = function() {
        function get_opt_by_id(option_group_selector) {
            var selected = $(option_group_selector + " .option.selected");
            return $(option_group_selector + " .option").hasClass("multi") ? (selected.sort(function(a, b) {
                return $(a).attr("sindex") - $(b).attr("sindex");
            }), $.map(selected, function(e) {
                return $(e).attr("id");
            })) : $(option_group_selector + " .option.selected").attr("id");
        }
        function get_opt_by_boolean(option_group_selector) {
            return $(option_group_selector + " .option.selected").hasClass("yes");
        }
        var unitsystem = get_opt_by_id(".unit_system");
        units = {}, $.each(units_categories, function() {
            this != "distance-lng" && this != "distance-shrt" ? units[this] = get_opt_by_id("#units ." + this) : (units.distance || (units.distance = {}), this == "distance-lng" ? units.distance.lng = get_opt_by_id("#units ." + this) : this == "distance-shrt" && (units.distance.shrt = get_opt_by_id("#units ." + this)));
        }), logger.debug("get UNITS: " + JSON.stringify(units)), display = {}, $.each(weather_data_categories, function() {
            display[this] = get_opt_by_boolean("#data_booleans ." + this);
        });
        var days = 7, show_english_locations = get_opt_by_boolean(".show_english_locations");
        days = parseInt(get_opt_by_id(".days").replace("days", ""));
        var icons = get_opt_by_id(".icon_pack");
        icons ? icons = icons.replace("icons-", "") : icons = previous_options.theme.icon_pack;
        var show_cc = get_opt_by_boolean(".show_cc"), show_radar = get_opt_by_id(".show_radar").replace("radar-", ""), radar_url = $("#radar_custom-url").val();
        radar_url.match(URL_REGEX) || (radar_url = "http://" + radar_url);
        var show_radar_custom = {
            url: radar_url,
            width: Number($("#radar_custom-width").val()),
            height: Number($("#radar_custom-height").val())
        }, show_swa = get_opt_by_boolean(".show_swa"), show_hourly = get_opt_by_boolean(".show_hourly"), show_day5 = get_opt_by_boolean(".show_day5"), rotate = $(".rotate_locations input[type=checkbox]").attr("checked"), rotate_interval = Number($(".rotate_locations input[type=text]").val()), show_days_or_nights = get_opt_by_id(".show_days_or_nights").replace("daynight-", "");
        if (rotate_interval == 0 || isNaN(rotate_interval)) rotate_interval = 1;
        var show_separators = get_opt_by_boolean(".show_separators"), label_display = parseInt(get_opt_by_id(".label_display").replace("labels", "")), location_name = $(".display_location input[type=checkbox]").attr("checked");
        return {
            unitsystem: unitsystem,
            units: units,
            display: display,
            show_english_locations: show_english_locations,
            defaults: !1,
            rotate: {
                enabled: rotate,
                interval: rotate_interval
            },
            theme: {
                icon_pack: icons,
                separators: show_separators
            },
            toolbar: {
                days: days,
                cc: show_cc,
                hourly: show_hourly,
                day5: show_day5,
                days_or_nights: show_days_or_nights,
                location_name: location_name,
                radar: show_radar,
                radar_custom: show_radar_custom,
                swa: show_swa,
                display: label_display,
                position: (previous_options.toolbar || {}).position,
                parent: (previous_options.toolbar || {}).parent || "status-bar"
            }
        };
    };
    function set_opt_by_id(option_group_selector, selected_option_id) {
        var options = $(option_group_selector + " .option").removeClass("selected");
        if (selected_option_id instanceof Array) {
            options.attr("sindex", "").children(".sindex").remove();
            for (var x = 0, len = selected_option_id.length; x < len; x++) options.filter("#" + selected_option_id[x]).addClass("selected").attr("sindex", x + 1).append('<span class="sindex">' + (x + 1) + "</span>");
        } else options.removeClass("selected"), options.filter("#" + selected_option_id).addClass("selected");
    }
    function set_opt_by_boolean(option_group_selector, bool) {
        var yesno = bool ? "yes" : "no";
        $(option_group_selector + " .option").toggleClass("selected", !1), $(option_group_selector + " .option." + yesno).toggleClass("selected", !0);
    }
    set_options = function(options) {
        options || (options = previous_options), previous_options = options;
        if ("version" in options) {
            var href = window.location.href, matches = href.match(/^http:\/\/www\.getforecastfox\.com\/customize\/([#]{0,1}(.*))$/i);
            if (matches.length >= 2) {
                var page_version = matches[1].replace("/", ""), our_version = options.version;
                page_version != our_version && (window.location.href += "../" + our_version + "/");
            }
        }
        "unitsystem" in options && set_opt_by_id(".unit_system", options.unitsystem), "rotate" in options && ($(".rotate_locations input[type=checkbox]").attr("checked", options.rotate.enabled), $(".rotate_locations input[type=text]").val(options.rotate.interval)), "units" in options && (logger.debug("set UNITS: " + JSON.stringify(options.units)), $.each(units_categories, function() {
            var name = this;
            name != "distance-lng" && name != "distance-shrt" ? name in options.units && set_opt_by_id("#units ." + name, options.units[this]) : (name == "distance-lng" && "distance" in options.units && "lng" in options.units.distance && set_opt_by_id("#units ." + name, options.units.distance.lng), name == "distance-shrt" && "distance" in options.units && "shrt" in options.units.distance && set_opt_by_id("#units ." + name, options.units.distance.shrt));
        })), "display" in options && $.each(weather_data_categories, function() {
            this in options.display && set_opt_by_boolean("#data_booleans ." + this, options.display[this]);
        }), "toolbar" in options && (set_opt_by_id(".days", "days" + options.toolbar.days), set_opt_by_boolean(".show_cc", options.toolbar.cc), set_opt_by_boolean(".show_swa", options.toolbar.swa), set_opt_by_id(".show_radar", "radar-" + options.toolbar.radar), $("#radar_custom-url").val(options.toolbar.radar_custom.url), options.toolbar.radar_custom.width ? $("#radar_custom-width").val(options.toolbar.radar_custom.width) : $("#radar_custom-width").val(null), options.toolbar.radar_custom.height ? $("#radar_custom-height").val(options.toolbar.radar_custom.height) : $("#radar_custom-height").val(null), options.toolbar.radar == "custom" ? $(".show_radar_custom").show() : $(".show_radar_custom").hide(), set_opt_by_boolean(".show_hourly", options.toolbar.hourly), set_opt_by_boolean(".show_day5", options.toolbar.day5), set_opt_by_id(".label_display", "labels" + options.toolbar.display), set_opt_by_id(".show_days_or_nights", "daynight-" + options.toolbar.days_or_nights), $(".display_location input[type=checkbox]").attr("checked", options.toolbar.location_name)), "theme" in options && (set_opt_by_id(".icon_pack", "icons-" + options.theme.icon_pack), set_opt_by_boolean(".show_separators", options.theme.separators)), "show_english_locations" in options && set_opt_by_boolean(".show_english_locations", options.show_english_locations), "unitsystem" in options && options.unitsystem == "custom" ? $(".custom_units").toggleClass("hidden", !1) : $(".custom_units").toggleClass("hidden", !0), "firefoxmigration" in options && options.firefoxmigration ? $("#migration-callout").toggleClass("hidden", !1) : $("#migration-callout").toggleClass("hidden", !0), options_initialized = !0;
    };
    var units_categories = [ "temperature", "speed", "time", "pressure", "distance-lng", "distance-shrt" ], weather_data_categories = [ "humidity", "precipitation", "pressure", "uv", "visibility", "wind", "quality", "moon" ], timeout;
    show_saved_message = function() {
        timeout && clearTimeout(timeout), $("#saved-callout").removeClass("hidden"), timeout = setTimeout(function() {
            $("#saved-callout").addClass("hidden");
        }, 1e3);
    };
    var icon_template = '<li class="option ${hidden}" id="icons-${id}"><div class="image ${icon}"/><div>${label}</div></li>';
    function initialize_icons() {
        var list_url = "http://www.getforecastfox.com/icons/all", container = $("#icon_pack_container");
        function init_html_packs() {
            radio_group(".icon_pack .option"), previous_options && previous_options.theme && set_opt_by_id(".icon_pack", "icons-" + previous_options.theme.icon_pack);
        }
        $.get(list_url, function(data) {
            var json = typeof data == "string" ? JSON.parse(data) : data, last_height = container.height(), selected = previous_options && previous_options.theme ? previous_options.theme.icon_pack : "", max = selected == 1 ? 7 : 6;
            for (var x = 0, len = json.length; x < len; x++) if (json[x].id == selected) {
                json = json.splice(x, 1).concat(json);
                break;
            }
            for (var x = 0, len = json.length; x < len; x++) {
                var icon_pack = json[x];
                container.append($.template(icon_template).apply({
                    label: icon_pack.name,
                    id: icon_pack.id,
                    icon: "ff-pack-preview-" + icon_pack.id,
                    hidden: x < max ? "" : "hidden"
                }));
            }
            init_html_packs();
        }), $("#theme .more").click(function() {
            return container.children(".option").removeClass("hidden"), $(this).hide(), init_html_packs(), !1;
        });
    }
    $(function() {
        radio_group("#units .unit_system .option"), $.each(units_categories, function() {
            var units_selector = "#units ." + this + " .option";
            radio_multi_group(units_selector);
        }), $.each(weather_data_categories, function() {
            var include_selector = "#data_booleans ." + this + " .option";
            radio_group(include_selector);
        }), radio_group(".days .option"), radio_group(".label_display .option"), $(".ui-tabs-nav").removeClass("ui-corner-all"), radio_group(".show_english_locations .option"), radio_group(".show_cc .option"), radio_group(".show_swa .option"), radio_group(".show_radar .option"), radio_group(".show_hourly .option"), radio_group(".show_day5 .option"), radio_group(".show_separators .option"), radio_group(".show_days_or_nights .option"), radio_group(".rotate_locations .option"), observers.add("ui-get-options-results", function(p) {
            set_options(p), initialize_icons();
        }), observers.add("preference-update", set_options), $(".option:not(.location)").live("click", function() {
            dispatcher.set_options(get_options());
        }), $("#migration-callout a").click(function() {
            return observers.notify("ui-set-options-request", {
                firefoxmigration: !1
            }), $("#migration-callout").toggleClass("hidden", !0), !1;
        }), $(".rotate_locations input[type=text]").live("change", function() {
            dispatcher.set_options(get_options());
        }), $(".show_radar_custom input[type=text]").live("change", function() {
            dispatcher.set_options(get_options());
        }), $("#drag-hint").tooltip({
            delay: 30,
            bodyHandler: function() {
                return $("#drag-hint-tooltip").html();
            },
            extraClass: "callout border-callout",
            left: -30,
            top: 15
        });
    });
})();